#!/usr/bin/env python

"""
Program code and data representations.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class Block:

    "A code block."

    def __init__(self):
        self.code = []
        self.location = None

    def __repr__(self):
        return "Block(%r, location=%r)" % (id(self), self.location)

    def as_raw(self, objtable, paramtable):
        for i, item in enumerate(self.code):
            if hasattr(item, "location"):
                item.location = location + i
        return self.code

class DataValue:

    "A representation of a raw program value."

    def __init__(self, context, ref):
        self.context = context
        self.ref = ref

    def __repr__(self):
        return "%r, %r" % (
            self.context, self.ref
            )

class DataObject:

    "A representation of a raw program data object."

    def __init__(self, classcode, attrcode, codeaddr, name, size, funccode=None):
        self.classcode = classcode
        self.attrcode = attrcode
        self.codeaddr = codeaddr
        self.name = name
        self.size = size
        self.funccode = funccode

    def with_size(self, size):
        return DataObject(self.classcode, self.attrcode, self.codeaddr, self.name, size, self.funccode)

    def with_callable(self, codeaddr):
        return DataObject(self.classcode, self.attrcode, codeaddr, self.name, self.size, self.funccode)

    def __repr__(self):
        return "%r # %s" % (
            (self.classcode, self.attrcode, self.codeaddr, self.funccode, self.size), self.name
            )

class FragmentObject:

    "A representation of a list fragment, used by list instances."

    def __init__(self, occupied_size, allocated_size):
        self.occupied_size = occupied_size
        self.allocated_size = allocated_size

    def __repr__(self):
        return "%r" % ((self.occupied_size, self.allocated_size),)

class Context:

    """
    A representation of a context used in a program, providing the special
    context value types.
    """

    def __init__(self, truth_value, repr):
        self.truth_value = truth_value
        self.repr = repr

    def __repr__(self):
        return self.repr

    def __nonzero__(self):
        return self.truth_value

    __shortrepr__ = __repr__

# A representation of a context that is replaced upon certain assignment
# operations.

ReplaceableContext = Context(0, "Replace")

# A representation of a context that is employed by classes so that
# instantiators have enough slots in the invocation frame, yet does not cause
# testing of the context or adjustment of the frame.

PlaceholderContext = Context(0, "Placeholder")

# vim: tabstop=4 expandtab shiftwidth=4
